
DynamicLinkLibrary variableByteSubclass: #TimerDLL
  classVariableNames: 
    ' callback '
  poolDictionaries: ''  !


!TimerDLL class methods !

callbackTable
	" Private - Returns the callback shape table for defining callback handlers."

	^#(
	"(callBackName (arguments...) returnType callingConvention)"

	( CbFuncPtr ( ulong ) none c )

	)!
  
constructEventsTriggered
	"Private - answer the set of events that instances of the
	receiver can trigger."
	^super constructEventsTriggered
 		add: #ticked ;
        yourself!
   
fileName
        "Answer the receiver's file name."
    ^'timer.dll'!
 
newCallBack: callBackName sending: selector to: aReceiver
	"Returns a CallBack object for sending message with selector to aReceiver.
	callBackName must be a valid name defined in callbackTable."

	| tuple aMessage parameterTypes |
	tuple := self callbackTable
		detect: [:one| one first = callBackName ]
		ifNone: [ ^self error: 'Invalid callback name.'].

	"check selector arguments with parameterTypes size"
	parameterTypes := tuple at: 2.
	(selector asString select: [:c| c = $: ]) size = parameterTypes size
	ifFalse: [ ^self error: 'Invalid selector (Must have ',parameterTypes size asString,' arguments.' ].

	aMessage := Message new
		receiver: aReceiver;
		selector: selector;
		yourself.
	^CallBack
		registerMessage: aMessage
		parameterTypes: parameterTypes
		returnType: (tuple at: 3)
		callingConvention: (tuple size > 3 ifTrue: [ tuple at: 4 ] ifFalse: [#api]).! !



!TimerDLL methods !

apiDisableCallbackFacility
	" Disable the callback facility."

	<api: DisableCallbackFacility none>
     ^self invalidArgument!
 
apiEnableCallbackFacility
	" Enable the callback facility."

	<api: EnableCallbackFacility none>
     ^self invalidArgument!

apiGetCallbackEnabled
	" Return the callback facility status."

	<api: GetCallbackEnabled long>
     ^self invalidArgument!
 
apiGetCallbackFunction
	" Get the address for the callback function."

	<api: GetCallbackFunction ulong>
     ^self invalidArgument!

apiGetResolution
	" Get timer resolution (ms)"

	<api: GetResolution long>
     ^self invalidArgument!
  
apiGetRunning
	" Get timer running status."

	<api: GetRunning boolean>
     ^self invalidArgument!
 
apiGetStartTime
	" Get timer start time."

	<api: GetStartTime ulong>
     ^self invalidArgument!
   
apiGetStopTime
	" Get timer stop time."

	<api: GetStopTime ulong>
     ^self invalidArgument!
  
apiGetTime
	" Get timer time."

	<api: GetTime ulong>
     ^self invalidArgument!
   
apiResetCount
	" Reset timer count."

	<api: ResetCount long>
     ^self invalidArgument!
   
apiResetTimer
	" Reset timer (full reset)."

	<api: ResetTimer none>
     ^self invalidArgument!

apiSetCallbackFunction: pCallback
	" Set the callback function to pCallback."

	<api: SetCallbackFunction ulong none>
     ^self invalidArgument!
   
apiSetResolution: anInteger
	" Set timer resolution (ms)"

	<api: SetResolution long long>
     ^self invalidArgument!
  
apiStartTimer
	" Start timer."

	<api: StartTimer long>
     ^self invalidArgument!
 
apiStopTimer
	" Stop timer."

	<api: StopTimer long>
     ^self invalidArgument!

callback

	^callback!
 
callback: aCallBack

	callback := aCallBack!
  
callbackEnabled
	" Return whether callback is enabled in the dll."

	^ self apiGetCallbackEnabled = 1!
   
callbackTo: anObject sending: aSelector
	" Sets the callback of the receiver to send message with aSelector to anObject.
	aSelector must be a message selector with no arguments."

	self callback: ( self class
		newCallBack: #CbFuncPtr
		sending: aSelector
		to: anObject )!

disableCallback
	" Disable the callback facility."

	^self apiDisableCallbackFacility!
   
enableCallback
	" Enable the callback facility."

	^self apiEnableCallbackFacility!
  
fullReset
	" Full reset timer parameters (stops timer)."

	^self apiResetTimer!
  
initialize
	" Initialize and enable callbacks."

	self initializeCallback; enableCallback!
   
initializeCallback
	" Initialize the callback for alerting the receiver."

	self callbackTo: self sending: #ticked:.
	self apiSetCallbackFunction: self callback asParameter!
   
printOn: aStream
	" Print the receiver on aStream."
	
	super printOn: aStream.
	aStream 
		nextPutAll: ' -> ';
		nextPutAll: self apiGetTime asExternalLong asInteger printString.!
   
releaseCallback
	" Release the callback of the receiver."

	self callback release!
   
reset
	" Reset timer count to zero."

	^self apiResetCount!
  
resolution
	" Return current resolution for the receiver's timer (ms)."

	^self apiGetResolution!

resolution: anInteger
	" Set current resolution for the receiver's timer (ms)."

	^self apiSetResolution: anInteger!
 
running

	^self apiGetRunning!

start

	^self apiStartTimer!
  
startTime

	^self apiGetStartTime asExternalLong asInteger!
   
stop

	^self apiStopTimer!

stopTime

	^self apiGetStopTime asExternalLong asInteger!
 
ticked: aParameter
	" Timer for the receiver just ticked."

	self 
		triggerEvent: #ticked 
		ifNotHandled: [ MessageBox warning: 'Timer #ticked event hasn''t been handled!!' ]
!

time
	" Return the current time for the receiver's timer"

	^self apiGetTime asExternalLong asInteger! !

